@@ -1,5 +1,5 @@
 /*
- * Copyright (C)2005-2012 Haxe Foundation
+ * Copyright (C)2005-2022 Haxe Foundation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -21,23 +21,33 @@
  */
 #include <neko.h>
 #include <string.h>
-#define PCRE_STATIC
-#include <pcre.h>
+
+#define PCRE2_CODE_UNIT_WIDTH 8
+
+#ifdef PCRE2_STATIC_LINK
+// this must be defined before loading the pcre header
+#define PCRE2_STATIC
+#endif
+#include <pcre2.h>
 
 #define PCRE(o)		((pcredata*)val_data(o))
 
 typedef struct {
+	// The compiled regex code
+	pcre2_code *regex;
+	// Number of capture groups
+	int n_groups;
+
+	// The last string matched
 	value str;
-	pcre *r;
-	int nmatchs;
-	int *matchs;
+	// Pointer to the allocated memory for match data
+	pcre2_match_data *match_data;
 } pcredata;
 
 DEFINE_KIND(k_regexp);
 
 static field id_pos;
 static field id_len;
-static pcre_extra limit;
 
 /**
 	<doc>
@@ -48,16 +58,18 @@ static pcre_extra limit;
 	</doc>
 **/
 
-static void free_regexp( value p ) {	
-	pcre_free( PCRE(p)->r );
+static void free_regexp( value p ) {
+	pcre2_code_free( PCRE(p)->regex );
+	pcre2_match_data_free( PCRE(p)->match_data );
 }
 
-static int do_exec( pcredata *d, const char *str, int len, int pos ) {
-	int res = pcre_exec(d->r,&limit,str,len,pos,0,d->matchs,d->nmatchs * 3);
+static int do_match( pcredata *d, const char *str, int len, int pos ) {
+	int res = pcre2_match(d->regex,str,len,pos,0,d->match_data,NULL);
 	if( res >= 0 )
 		return 1;
-	if( res != PCRE_ERROR_NOMATCH )
-		val_throw(alloc_string("An error occured while running pcre_exec"));
+	d->str = val_null; // empty string prevents trying to access the data after a failed match
+	if( res != PCRE2_ERROR_NOMATCH )
+		val_throw(alloc_string("An error occurred while running pcre2_match"));
 	return 0;
 }
 
@@ -78,53 +90,55 @@ static value regexp_new_options( value s, value opt ) {
 	val_check(opt,string);
 	{
 		value v;
-		const char *error;
-		int err_offset;
-		pcre *p;
+		int error_num;
+		size_t err_offset;
+		pcre2_code *p;
 		pcredata *pdata;
 		char *o = val_string(opt);
 		int options = 0;
 		while( *o ) {
 			switch( *o++ ) {
 			case 'i':
-				options |= PCRE_CASELESS;
+				options |= PCRE2_CASELESS;
 				break;
 			case 's':
-				options |= PCRE_DOTALL;
+				options |= PCRE2_DOTALL;
 				break;
 			case 'm':
-				options |= PCRE_MULTILINE;
+				options |= PCRE2_MULTILINE;
 				break;
 			case 'u':
-				options |= PCRE_UTF8;
+				options |= PCRE2_UTF;
 				break;
 			case 'g':
-				options |= PCRE_UNGREEDY;
+				options |= PCRE2_UNGREEDY;
 				break;
 			default:
 				neko_error();
 				break;
 			}
 		}
-		p = pcre_compile(val_string(s),options,&error,&err_offset,NULL);
+		p = pcre2_compile(val_string(s),val_strlen(s),options,&error_num,&err_offset,NULL);
 		if( p == NULL ) {
 			buffer b = alloc_buffer("Regexp compilation error : ");
-			buffer_append(b,error);
+			PCRE2_UCHAR error_buffer[256];
+			pcre2_get_error_message(error_num,error_buffer,sizeof(error_buffer));
+			buffer_append(b,error_buffer);
 			buffer_append(b," in ");
 			val_buffer(b,s);
 			bfailure(b);
 		}
 		v = alloc_abstract(k_regexp,alloc(sizeof(pcredata)));
 		pdata = PCRE(v);
-		pdata->r = p;
+		pdata->regex = p;
 		pdata->str = val_null;
-		pdata->nmatchs = 0;
-		pcre_fullinfo(p,NULL,PCRE_INFO_CAPTURECOUNT,&pdata->nmatchs);
-		pdata->nmatchs++;
-		pdata->matchs = (int*)alloc_private(sizeof(int) * 3 * pdata->nmatchs);
+		pdata->n_groups = 0;
+		pcre2_pattern_info(pdata->regex,PCRE2_INFO_CAPTURECOUNT,&pdata->n_groups);
+		pdata->n_groups++;
+		pdata->match_data = pcre2_match_data_create_from_pattern(pdata->regex,NULL);
 		val_gc(v,free_regexp);
 		return v;
-	}	
+	}
 }
 
 /**
@@ -152,7 +166,7 @@ static value regexp_match( value o, value s, value p, value len ) {
 	if( pp < 0 || ll < 0 || pp > val_strlen(s) || pp + ll > val_strlen(s) )
 		neko_error();
 	d = PCRE(o);
-	if( do_exec(d,val_string(s),ll+pp,pp) ) {
+	if( do_match(d,val_string(s),ll+pp,pp) ) {
 		d->str = s;
 		return val_true;
 	} else {
@@ -161,22 +175,25 @@ static value regexp_match( value o, value s, value p, value len ) {
 	}
 }
 
-static value do_replace( value o, value s, value s2, bool all ) {	
-	val_check_kind(o,k_regexp);	
+static value do_replace( value o, value s, value s2, bool all ) {
+	val_check_kind(o,k_regexp);
 	val_check(s,string);
-	val_check(s2,string);	
+	val_check(s2,string);
 	{
 		pcredata *d = PCRE(o);
 		buffer b = alloc_buffer(NULL);
-		int pos = 0;
-		int len = val_strlen(s);
 		const char *str = val_string(s);
 		const char *str2 = val_string(s2);
+		int len = val_strlen(s);
 		int len2 = val_strlen(s2);
-		while( do_exec(d,str,len,pos) ) {
-			buffer_append_sub(b,str+pos,d->matchs[0] - pos);
+		int pos = 0;
+		size_t *matches;
+		// TODO: Consider pcre2_substitute()
+		while( do_match(d,str,len,pos) ) {
+			matches = pcre2_get_ovector_pointer(d->match_data);
+			buffer_append_sub(b,str+pos,matches[0] - pos);
 			buffer_append_sub(b,str2,len2);
-			pos = d->matchs[1];
+			pos = matches[1];
 			if( !all )
 				break;
 		}
@@ -190,7 +207,7 @@ static value do_replace( value o, value s, value s2, bool all ) {
 	regexp_replace : 'regexp -> from:string -> by:string -> string
 	<doc>Perform a replacement using a regexp</doc>
 **/
-static value regexp_replace( value o, value s, value s2 ) {	
+static value regexp_replace( value o, value s, value s2 ) {
 	return do_replace(o,s,s2,false);
 }
 
@@ -216,11 +233,13 @@ static value regexp_replace_fun( value o, value s, value f ) {
 		int pos = 0;
 		int len = val_strlen(s);
 		const char *str = val_string(s);
+		size_t *matches;
 		d->str = s;
-		while( do_exec(d,str,len,pos) ) {
-			buffer_append_sub(b,str+pos,d->matchs[0] - pos);
+		while( do_match(d,str,len,pos) ) {
+			matches = pcre2_get_ovector_pointer(d->match_data);
+			buffer_append_sub(b,str+pos,matches[0] - pos);
 			val_buffer(b,val_call1(f,o));
-			pos = d->matchs[1];
+			pos = matches[1];
 		}
 		d->str = val_null;
 		buffer_append_sub(b,str+pos,len-pos);
@@ -230,21 +249,22 @@ static value regexp_replace_fun( value o, value s, value f ) {
 
 /**
 	regexp_matched : 'regexp -> n:int -> string?
-	<doc>Return the [n]th matched block by the regexp. If [n] is 0 then return 
+	<doc>Return the [n]th matched block by the regexp. If [n] is 0 then return
 	the whole matched substring. If the [n]th matched block was optional and not matched, returns null</doc>
 **/
 static value regexp_matched( value o, value n ) {
 	pcredata *d;
 	int m;
-	val_check_kind(o,k_regexp);	
+	val_check_kind(o,k_regexp);
 	d = PCRE(o);
 	val_check(n,int);
 	m = val_int(n);
-	if( m < 0 || m >= d->nmatchs || val_is_null(d->str) )
+	if( m < 0 || m >= d->n_groups || val_is_null(d->str) )
 		neko_error();
 	{
-		int start = d->matchs[m*2];
-		int len = d->matchs[m*2+1] - start;
+		size_t *matches = pcre2_get_ovector_pointer(d->match_data);
+		int start = matches[m*2];
+		int len = matches[m*2+1] - start;
 		value str;
 		if( start == -1 )
 			return val_null;
@@ -262,15 +282,16 @@ static value regexp_matched( value o, value n ) {
 static value regexp_matched_pos( value o, value n ) {
 	pcredata *d;
 	int m;
-	val_check_kind(o,k_regexp);	
+	val_check_kind(o,k_regexp);
 	d = PCRE(o);
 	val_check(n,int);
 	m = val_int(n);
-	if( m < 0 || m >= d->nmatchs || val_is_null(d->str) )
+	if( m < 0 || m >= d->n_groups || val_is_null(d->str) )
 		neko_error();
 	{
-		int start = d->matchs[m*2];
-		int len = d->matchs[m*2+1] - start;
+		size_t *matches = pcre2_get_ovector_pointer(d->match_data);
+		int start = matches[m*2];
+		int len = matches[m*2+1] - start;
 		value o = alloc_object(NULL);
 		alloc_field(o,id_pos,alloc_int(start));
 		alloc_field(o,id_len,alloc_int(len));
@@ -278,11 +299,23 @@ static value regexp_matched_pos( value o, value n ) {
 	}
 }
 
+/**
+	regexp_matched_num : 'regexp -> int
+	<doc>Return the total number of matched groups, or -1 if the regexp has not
+	been matched yet</doc>
+**/
+static value regexp_matched_num( value o ) {
+	pcredata *d;
+	val_check_kind(o,k_regexp);
+	d = PCRE(o);
+	if( val_is_null(d->str) )
+		return alloc_int(-1);
+	return alloc_int(d->n_groups);
+}
+
 void regexp_main() {
 	id_pos = val_id("pos");
-	id_len = val_id("len");	
-	limit.flags = PCRE_EXTRA_MATCH_LIMIT_RECURSION;
-	limit.match_limit_recursion = 3500; // adapted based on Windows 1MB stack size
+	id_len = val_id("len");
 }
 
 DEFINE_PRIM(regexp_new,1);
@@ -293,6 +326,7 @@ DEFINE_PRIM(regexp_replace_all,3);
 DEFINE_PRIM(regexp_replace_fun,3);
 DEFINE_PRIM(regexp_matched,2);
 DEFINE_PRIM(regexp_matched_pos,2);
+DEFINE_PRIM(regexp_matched_num,1);
 DEFINE_ENTRY_POINT(regexp_main);
 
 /* ************************************************************************ */
